/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integraci�n de Tecnolog�as SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache;

import es.prodevelop.gvsig.mini.common.IEvent;

/**
 * This interface is used to get notified of the download process. An
 * implementation of this interface should update for example the UI during the
 * download process
 * 
 * @author aromeu
 */
public interface IDownloadWaiter extends IDownloadCallbackHandler {
	
	/**
	 * This event is thrown when the TileProvider is in MODE_OFFLINE and the tile is not in the file system
	 * 
	 * @param event The TileEvent
	 */
	public void onTileNotFound(IEvent event);
	
	/**
	 * This event is thrown after every single tile has been deleted and the TileProvider is in MODE_DELETE
	 * 
	 * @param event The TileEvent
	 */
	public void onTileDeleted(IEvent event);

	/**
	 * This event is thrown after every single tile has been downloaded
	 * 
	 * @param event The TileEvent
	 */
	public void onTileDownloaded(IEvent event);

	/**
	 * This event is thrown when the tile either nor intersects with the
	 * ITileBufferIntersector geometry or is outside the bounds of the map
	 * 
	 * @param event The TileEvent
	 */
	public void onTileSkipped(IEvent event);

	/**
	 * The first event thrown
	 * 
	 * @param totalNumTiles
	 *            - The total number tiles to be downloaded
	 */
	public void onTotalNumTilesRetrieved(final long totalNumTiles);

	/**
	 * This event is thrown when no more tiles left to be downloaded
	 */
	public void onFinishDownload();

	/**
	 * This event indicates the download proceess is going to start
	 */
	public void onStartDownload();

	/**
	 * This event indicates that the process has benn cancelled
	 * 
	 * @see Cancellable
	 */
	public void onDownloadCanceled();

	/**
	 * Thrown when the download of a tile has failed
	 * 
	 * @param event The TileEvent
	 */
	public void onFailDownload(IEvent event);

	/**
	 * Thrown when an OutOfMemoryError happens
	 * 
	 * @param event The TileEvent
	 */
	public void onFatalError(IEvent event);

	/**
	 * This event indicates that an amount of bytes have been downloaded
	 * 
	 * @param size
	 *            The size of the tile downlaoded. Size_in_bytes = size/1024
	 */
	public void updateDataTransfer(int size);

	/**
	 * This event is thrown when the tile is not downloaded from the server
	 * because was previously stored on the file system
	 * 
	 * @param event The TileEvent
	 */
	public void onTileLoadedFromFileSystem(IEvent event);
}
