/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Alberto Romeu aromeu@prodevelop.es 
 *   author Ruben Blanco rblanco@prodevelop.es 
 *   
 */

package es.prodevelop.tilecache.layers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.crypto.EncryptedPrivateKeyInfo;

import es.prodevelop.gvsig.mini.common.CompatManager;
import es.prodevelop.gvsig.mini.common.IContext;
import es.prodevelop.gvsig.mini.common.IEncryptor;
import es.prodevelop.gvsig.mini.exceptions.BaseException;
import es.prodevelop.gvsig.mini.utiles.Constants;
import es.prodevelop.gvsig.mini.utiles.LayersSorter;
import es.prodevelop.tilecache.renderer.MapRenderer;
import es.prodevelop.tilecache.renderer.MapRendererManager;

/**
 * Class to manage the persistence of Layers. It should feed the UI
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class Layers {

	private static final Logger logger = Logger.getLogger("Layers");

	private static Layers instance;

	private Hashtable properties;
	private Hashtable<Integer, Vector> layers;

	private final static String fileName = "layers.txt";
	private LayersSorter mLayersSorter;

	private IContext context;

	/**
	 * A singleton. Creates a new instance of layers and fill the properties
	 * with the info at
	 * Environment.getExternalStorageDirectory().getPath()).append(
	 * File.separator).append(Utils.APP_DIR).append(File.separator)
	 * .append(Utils.LAYERS_DIR).append(File.separator).append(
	 * "layers.txt").toString()
	 * 
	 * @return
	 */
	public static Layers getInstance() {
		try {
			if (instance == null) {
				instance = new Layers();
				instance.setLayersSorter(new LayersSorter());
				try {
					CompatManager.getInstance().getRegisteredLogHandler()
							.configureLogger(logger);
				} catch (BaseException e) {

				} catch (Exception e) {

				}
			}
			return instance;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getInstance: ", e);
			return null;
		}
	}

	public IContext getContext() throws BaseException {
		if (context == null)
			context = CompatManager.getInstance().getRegisteredContext();
		return context;
	}

	public void initialize(boolean persist) throws BaseException {
		try {
			File layerFile = new File(getContext().getBaseLayerFilePath());
			boolean exists = layerFile.exists();
			loadProperties(getContext().getBaseLayerFilePath());
			if (!exists && persist)
				instance.persist();
		} catch (BaseException e) {
			logger.log(Level.SEVERE, "IOException on load layers", e);
			loadProperties(null);
		}
	}

	/**
	 * Parses a layer configuration file.
	 * 
	 * @param filePath
	 *            The path of the layer configuration file to load. If this
	 *            parameter is null then the 'layers.txt' file in the apk assets
	 *            will be loaded
	 * @throws Exception
	 */
	public void loadProperties(String filePath) throws BaseException {
		BufferedReader reader = null;
		InputStream is = null;
		File layerFile = null;
		try {
			logger.log(Level.FINE, "loadProperties");
			if (filePath == null) {
				logger.log(Level.FINE, "filePath is null");
				properties = new Hashtable();
				layers = new Hashtable();
				layerFile = new File(getContext().getBaseLayerFilePath());
				boolean exists = layerFile.exists();
				if (exists) {
					logger.log(Level.FINE, layerFile.getAbsolutePath());
					loadProperties(getContext().getBaseLayerFilePath());
					return;
				} else {
					logger.log(Level.FINE, "load layers.txt from assets");
					is = getContext().openAssetFile("layers.txt");
				}
			} else {
				logger.log(Level.FINE, "filePath: " + filePath);
				// if (properties == null)
				properties = new Hashtable();
				// if (layers == null)
				layers = new Hashtable();

				layerFile = new File(filePath);
				if (!layerFile.exists()) {
					logger.log(Level.FINE, "layerFile not exists");
					loadProperties(null);
					return;
					// throw new IOException("File " + filePath
					// + " does not exist.");
				}

				is = new FileInputStream(layerFile);
			}

			reader = new BufferedReader(new InputStreamReader(is));
			boolean failed = parseLayersFile(reader);
			if (failed) {
				logger.log(Level.FINE,
						"parse layers file failed. Invalid version");
				instance.persist("layersback.txt");
				logger.log(Level.FINE, "previous layer file persisted");
				// try {
				// File layerFile_ = new File(layerFile.getAbsolutePath()+"_");
				// if (layerFile.exists() || layerFile_.createNewFile()) {
				// FileOutputStream out = new FileOutputStream(layerFile_);
				// Utils.copy(is, out);
				// out.flush();
				// }
				// } catch (Exception e) {
				// logger.error(e);
				// }
				logger.log(Level.FINE, "loading last layers file version");
				loadLayersAssets();
				instance.persist();
				logger.log(Level.FINE, "last layers file version persisted");
			}

		} catch (IOException e) {
			throw new BaseException(e.getCause());
		} catch (Exception e) {
			logger.log(Level.SEVERE, "loadProperties: ", e);
			throw new BaseException(e.getCause());
		} finally {
			Constants.closeStream(is);
		}
	}

	private void loadLayersAssets() throws Exception {
		BufferedReader reader = null;
		InputStream is = null;
		try {

			logger.log(Level.FINE, "load layers.txt from assets");
			is = getContext().openAssetFile("layers.txt");

			reader = new BufferedReader(new InputStreamReader(is));
			parseLayersFile(reader);

		} catch (IOException e) {
			throw e;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "loadProperties: ", e);
			throw e;
		} finally {
			Constants.closeStream(is);
		}
	}

	/**
	 * Clears the properties Hashtable
	 */
	public void clearProperties() {
		try {
			logger.log(Level.FINE, "clearProperties");
			properties = new Hashtable();
			layers = new Hashtable();
		} catch (Exception e) {
			logger.log(Level.SEVERE, "clearProperties", e);
		}
	}

	/**
	 * Adds a layer to the current properties. The layer String must be in the
	 * format defined by the layers.txt file: OPEN STREET
	 * MAP;0,http://a.tile.openstreetmap.org/,png,17,256
	 * 
	 * @param layer
	 */
	public void addLayer(String layer) {
		try {
			try {
				layer = CompatManager.getInstance()
				.getRegisteredEncIEncryptor().tryDecrypt(layer);
			} catch (Exception e) {
				
			}

			if (!layer.contains(";"))
				return;
			String[] part = layer.split(";");
			properties.put(part[0], part[1]);
			Integer in = new Integer(part[1].substring(0, 1));
			if (in >= MapRenderer.OSM_RENDERER && in < MapRenderer.WMS_RENDERER
					|| (in >= MapRenderer.OSM_ZYX_RENDERER)) {
				if (part[0].contains(MapRenderer.NAME_SEPARATOR))
					in = new Integer(2);
				else
					in = new Integer(0);
			} else {
				if (part[0].contains(MapRenderer.NAME_SEPARATOR))
					in = new Integer(2);
				else
					in = new Integer(1);
			}
			Vector v = layers.get(in);
			if (v == null) {
				v = new Vector();
			}
			v.add(part[0]);

			layers.put(in, v);

			logger.log(Level.FINE, "Found: " + part[0] + " with value: "
					+ part[1]);
		} catch (Exception e) {
			logger.log(Level.SEVERE, "addLayer: ", e);
		}
	}

	/**
	 * Reads the BufferedReader line to line and calls {@link #addLayer(String)}
	 * 
	 * @param reader
	 * @return false if the version of the file is not correct
	 */
	public boolean parseLayersFile(final BufferedReader reader) {
		boolean failed = false;
		try {
			String line = null;
			int i = 0;
			while ((line = reader.readLine()) != null) {
				if (i == 0) {
					if (line.compareToIgnoreCase(Constants.LAYERS_VERSION) != 0) {
						failed = true;
					}
				}
				i++;
				if (i != 0)
					addLayer(line);
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "parseLayersFile: ", e);
			failed = true;
		} finally {
			Constants.closeStream(reader);
			return failed;
		}
	}

	public Hashtable getLayers() {
		return properties;
	}

	private String getLayerKeyFromName(String layerName) {
		try {
			Enumeration keys = properties.keys();

			String key = null;
			String temp = null;
			while (keys.hasMoreElements()) {
				try {
					key = keys.nextElement().toString();
					temp = key
							.substring(key.lastIndexOf("|") + 1, key.length());
					if (temp.equals(layerName))
						return key;
				} catch (Exception ignore) {

				}
			}
			return layerName;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "layerkeyname", e);
			return null;
		}
	}

	/**
	 * Instantiates a MapRenderer
	 * 
	 * @param layerTitle
	 *            The title of the layer
	 * @return A MapRenderer
	 * @throws IOException
	 *             If the layers file has a bad format
	 */
	public MapRenderer getRenderer(String layerTitle) throws IOException {
		MapRenderer renderer = null;
		try {
			String layer = properties.get(getLayerKeyFromName(layerTitle))
					.toString();
			String[] layerProps = layer.split(",");
			final int size = layerProps.length;

			if (layerProps.length == 0) {
				logger.log(Level.FINE, "Bad layers file!");
				throw new IOException("Bad layers file");
			} else {
				renderer = MapRendererManager.getInstance()
						.getMapRendererFactory()
						.getMapRenderer(layerTitle, layerProps);

				return renderer;
			}
		} catch (IOException ioe) {
			throw ioe;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getRenderer: ", e);
			return renderer;
		}
	}

	public Hashtable getLayersForView() {
		try {
			Enumeration keys = layers.keys();

			Integer key = null;
			Vector keyLayers = null;
			while (keys.hasMoreElements()) {
				key = (Integer) keys.nextElement();
				keyLayers = layers.get(key);
				keyLayers = (Vector) mLayersSorter.sort(keyLayers);
				layers.put(key, keyLayers);
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getLayersforview", e);
		}
		return layers;
	}

	/**
	 * Persists the Layers properties into a file in the provided directory
	 * 
	 * @param directoryAbsolutPath
	 *            The path of the directory where the file is going to be
	 *            persisted
	 * @param fileName
	 *            The fileName
	 */
	public synchronized void persist(String directoryAbsolutPath,
			String fileName) {
		BufferedWriter out = null;
		FileWriter logwriter = null;
		try {
			if (properties != null) {
				String dirPath = directoryAbsolutPath + File.separator;

				File f = new File(dirPath + fileName);
				if (!f.exists()) {
					File dirFile = new File(dirPath);
					dirFile.mkdirs();
					f.createNewFile();
				} else {
					f.delete();
					f.createNewFile();
				}
				logwriter = new FileWriter(f, true);
				out = new BufferedWriter(logwriter);

				Enumeration keys = properties.keys();

				String layerTitle = null;
				MapRenderer renderer = null;

				out.write(Constants.LAYERS_VERSION + "\n");
				while (keys.hasMoreElements()) {
					try {
						layerTitle = keys.nextElement().toString();
						if (layerTitle != null) {
							renderer = Layers.getInstance().getRenderer(
									layerTitle);
							if (renderer != null) {
								logger.log(Level.FINE, layerTitle
										+ " persisted");
								// FIXME ENCRYPTION
								String persist = renderer.toString();
								if (renderer.isOffline()) {
									persist = encryptLayer(persist);
								}
								out.write(persist + "\n");
							}
						}
					} catch (Exception e) {
						logger.log(Level.SEVERE, "error while writing: "
								+ layerTitle);
					}
				}
			}

		} catch (Exception e) {
			logger.log(Level.SEVERE, "persist: ", e);
		} finally {
			Constants.closeStream(out);
			Constants.closeStream(logwriter);
		}
	}

	/**
	 * Persists the Layers properties into a file in the directory: SDDIR +
	 * File.separator + Constants.APP_DIR + File.separator +
	 * Constants.LAYERS_DIR
	 * 
	 * @param fileName
	 *            The fileName
	 */
	public void persist(String fileName) {
		BufferedWriter out = null;
		FileWriter logwriter = null;
		try {
			if (properties != null) {
				String dirPath = getContext().getBaseLayerPersistFilePath();

				File dir = new File(dirPath).getParentFile();
				File f = new File(dir.getAbsolutePath() + File.separator
						+ fileName);
				if (!f.exists()) {
					f.getParentFile().mkdirs();
					f.createNewFile();
				} else {
					f.delete();
					f.createNewFile();
				}
				logwriter = new FileWriter(f, true);
				out = new BufferedWriter(logwriter);

				Enumeration keys = properties.keys();

				String layerTitle = null;
				MapRenderer renderer = null;

				out.write(Constants.LAYERS_VERSION + "\n");
				while (keys.hasMoreElements()) {
					try {
						layerTitle = keys.nextElement().toString();
						if (layerTitle != null) {
							renderer = Layers.getInstance().getRenderer(
									layerTitle);
							if (renderer != null) {
								logger.log(Level.FINE, layerTitle
										+ " persisted");
								String persist = renderer.toString();
								if (renderer.isOffline()) {
									persist = encryptLayer(persist);
								}
								out.write(persist + "\n");
							}
						}
					} catch (Exception e) {
						logger.log(Level.SEVERE, "error while writing: "
								+ layerTitle);
					}
				}
			}

		} catch (Exception e) {
			logger.log(Level.SEVERE, "persist: ", e);
		} finally {
			Constants.closeStream(out);
			Constants.closeStream(logwriter);
		}
	}
	
	private String encryptLayer(String layer) throws BaseException, Exception {
		layer = CompatManager.getInstance()
		.getRegisteredEncIEncryptor()
		.encrypt(layer, CompatManager.getInstance()
				.getRegisteredEncIEncryptor().getUniqueID());
		layer = IEncryptor.DEV_ID + layer;
		return layer;
	}

	/**
	 * Persists the layers properties to the default fileName (layers.txt)
	 */
	public void persist() {
		persist(fileName);
	}

	/**
	 * Sets the LayersSorter
	 * 
	 * @param aLayersSorter
	 */
	public void setLayersSorter(LayersSorter aLayersSorter) {
		this.mLayersSorter = aLayersSorter;
	}
}
