package es.prodevelop.tilecache.provider.android;

import es.prodevelop.gvsig.mini.common.IBitmap;
import es.prodevelop.gvsig.mini.common.IContext;
import es.prodevelop.gvsig.mini.common.IHandler;
import es.prodevelop.gvsig.mini.common.impl.Tile;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.tilecache.draw.TileDrawer;
import es.prodevelop.tilecache.provider.Downloader;
import es.prodevelop.tilecache.provider.TileProvider;
import es.prodevelop.tilecache.provider.filesystem.impl.BackgroundFileSystemProvider;
import es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy;

public class AndroidTileProvider extends TileProvider {

	public AndroidTileProvider(IContext ctx, IHandler loadCallbackHandler,
			int width, int height, int tileSize, int mode,
			ITileFileSystemStrategy fileSystemStrategy) {
		super(ctx, loadCallbackHandler, width, height, tileSize, mode,
				fileSystemStrategy);
		this.mFSTileProvider = new BackgroundFileSystemProvider(
				4 * 1024 * 1024, this.mTileCache, ctx, fileSystemStrategy,
				mode, this.mOfflineMapTile, this);
		this.mTileDownloader = new Downloader(ctx, this.mFSTileProvider);
	}

	/**
	 * The strategy to get Tiles is: If the tile is in memory cache, then is
	 * returned and put the first (LRU strategy) If not then the tile is
	 * searched on the FileSystem if the tile is found, is put into the memory
	 * cache if not then is downloaded from the server, save to file system and
	 * finally put into memroy cache
	 * 
	 * @param tile
	 *            The tile containing the tile URL and the tile x-y, zoomLevel
	 *            and renderer name
	 * @param cancellable
	 *            A Cancellable instance
	 * @param drawer
	 *            A TileDrawer with a bufferCanvas to draw the tiles also in
	 *            background
	 * @return A Bitmap representing the tile
	 */
	public IBitmap getMapTile(final Tile tile, final Cancellable cancellable,
			final TileDrawer drawer) {
//		long t1 = System.currentTimeMillis();
		try {
			boolean returnLoadingTile = false;
			
			String cacheURL = tile.getTileString();
			IBitmap ret = null;
			if (mTileCache != null)
				ret = this.mTileCache.getMapTile(cacheURL);
			// long t2 = System.currentTimeMillis();
			// System.out.println("from cache: " + (t2 - t1) + " ms");
			if (ret != null) {
				// System.out.println("Load from cache: " + cacheURL);
				mCtx.sendMessage(mLoadCallbackHandler, this
						.instantiateEvent(tile), MAPTILECACHE_SUCCESS_ID);
				if (drawer != null) {
					drawer.drawTile(tile.clone(), ret);
				}
				return ret;
			} else {
				final boolean prepend = lastZoomLevelRequested == tile.zoomLevel;
				if (lastZoomLevelRequested == -1 || prepend
						) {
					returnLoadingTile = true;
					this.setViewExtent(null);
				}
				// this.lastZoomLevelRequested = tile.zoomLevel;
				// log.log(Level.FINE, "Cache failed, trying from FS.");
				try {
					// t1 = System.currentTimeMillis();
					if (this.mCtx.isSDMounted()) {
						if (!this.mFSTileProvider.getPendingQueue().contains(
								cacheURL)) {
							// log.log(Level.FINE, "load from disk");
							if (this.mFSTileProvider
									.loadMapTileToMemCacheAsync(tile,
											this.mLoadCallbackHandler,
											cacheURL, cancellable, drawer,
											prepend)) {
//								System.out.println("Load from disk: "
//										+ cacheURL);
								if (this.getMode() == TileProvider.MODE_OFFLINE)
									if (returnLoadingTile) {
//										System.out.println("a");
										ret = this.mOfflineMapTile;
									} else {
//										System.out.println("b");
										ret = mOfflineMapTile;
									}
								else if (returnLoadingTile) {
//									System.out.println("c");
									ret = this.mLoadingMapTile;
								} else {
//									System.out.println("d");
									ret = mLoadingMapTile;
								}
								// t2 = System.currentTimeMillis();
								// System.out.println("from fs: " + (t2 - t1)
								// + " ms");
							} else {
//								System.out.println("Load from server 0: "
//										+ cacheURL);
								ret = this.downloadFromServer(tile, cacheURL,
										cancellable, drawer, prepend);
							}
						} else {

							if (this.getMode() == TileProvider.MODE_OFFLINE)
								ret = this.mOfflineMapTile;
							else
								ret = this.mLoadingMapTile;

						}
					} else {
//						System.out.println("Load from server 1: " + cacheURL);
						ret = this.downloadFromServer(tile, cacheURL,
								cancellable, drawer, prepend);
					}
				} catch (Exception e) {
//					System.out.println("Load from server 2: " + cacheURL);
					ret = this.downloadFromServer(tile, cacheURL, cancellable,
							drawer, prepend);
					e.printStackTrace();

					// log.log(Level.SEVERE, e);
				}
				// if (ret == null) {
				//									
				// }
			}
			// if (ret == this.mLoadingMapTile) return null;
			if (returnLoadingTile)
				return ret;
			else
				return ret;

		} catch (Exception e) {
			// log.log(Level.SEVERE, "", e);
			return this.mLoadingMapTile;
		} finally {
//			System.out.println("Total time to request a tile: "
//					+ (System.currentTimeMillis() - t1));
		}
	}

}
