/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Rub�n Blanco rblanco@prodevelop.es
 *
 *
 * Original version of the code made by Nicolas Gramlich.
 * No header license code found on the original source file.
 * 
 * Original source code downloaded from http://code.google.com/p/osmdroid/
 * package org.andnav.osm.views.util;
 * 
 * License stated in that website is 
 * http://www.gnu.org/licenses/lgpl.html
 * As no specific LGPL version was specified, LGPL license version is LGPL v2.1
 * is assumed.
 *  
 * 
 */

package es.prodevelop.tilecache.provider.filesystem.impl;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import es.prodevelop.gvsig.mini.common.IBitmap;
import es.prodevelop.gvsig.mini.common.IContext;
import es.prodevelop.gvsig.mini.common.IHandler;
import es.prodevelop.gvsig.mini.common.impl.Tile;
import es.prodevelop.gvsig.mini.map.GeoUtils;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.gvsig.mini.utiles.Constants;
import es.prodevelop.gvsig.mini.utiles.WorkQueue;
import es.prodevelop.tilecache.draw.TileDrawer;
import es.prodevelop.tilecache.provider.Downloader;
import es.prodevelop.tilecache.provider.TileCache;
import es.prodevelop.tilecache.provider.TileProvider;
import es.prodevelop.tilecache.provider.event.TileEvent;
import es.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider;
import es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy;

/**
 * A provider of tiles stored on the file system
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class TileFilesystemProvider implements GeoUtils,
		ITileFileSystemProvider {

	String datalog = null;
	Downloader down;
	// public byte[] datas = null;
	// protected ExecutorService mThreadPool = Executors.newFixedThreadPool(2);
	public final TileCache mCache;
	public HashSet<String> mPending = new HashSet<String>();
	static String SD_DIR;
	private final static Logger log = Logger
			.getLogger("TileFilesystemProvider");
	IContext context;
	ITileFileSystemStrategy strategy;
	int mode;
	IBitmap offlineBitmap;

	/**
	 * The constructor
	 * 
	 * @param aMaxFSCacheByteSize
	 *            Not used at the moment
	 * @param aCache
	 *            The TileCache instance
	 * @param IContext
	 *            a common interface which instances implement methods that are
	 *            incompatible among Java versions (Android, J2SE, etc.)
	 * @param The
	 *            strategy to store tiles on system
	 * @param mode
	 *            MODE_ONLINE = 0 - Gets tiles from file system and from the
	 *            server MODE_OFFLINE = 1 - Only gets tiles that are on file
	 *            system MODE_UPDATE = 2 - If a tile is on the file system,
	 *            deletes it and tries to download MODE_DELETE = 3 - Deletes the
	 *            tile cache MODE_CONNECTED = 4 - Don't save any tile, always
	 *            download them
	 * @param mode
	 *            offlineBitmap The Bitmap to show when a tile is not found when
	 *            offline_mode is on
	 */
	public TileFilesystemProvider(final int aMaxFSCacheByteSize,
			final TileCache aCache, final IContext context,
			final ITileFileSystemStrategy strategy, final int mode,
			final IBitmap offlineBitmap) {
		this.mCache = aCache;
		try {
			this.offlineBitmap = offlineBitmap;
			// try {
			// CompatManager.getInstance().getRegisteredLogHandler().
			// configureLogger(log);
			// } catch (BaseException e) {
			//				
			// }
			this.strategy = strategy;
			this.mode = mode;
			this.context = context;
		} catch (Exception e) {
			log.log(Level.SEVERE, "", e);
		}
	}

	protected final String getSDDir() {
		try {			
			SD_DIR = context.getExternalStorageDirectoryPath();
			return SD_DIR;
		} catch (Exception e) {
			log.log(Level.SEVERE, "", e);
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seees.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider#
	 * loadMapTileToMemCacheAsync(java.lang.String,
	 * es.prodevelop.gvsig.mini.common.IHandler, int[], java.lang.String, int,
	 * java.lang.String)
	 */
	public synchronized boolean loadMapTileToMemCacheAsync(final Tile tile,
			final IHandler callback, final String cacheURL,
			final Cancellable cancellable, final TileDrawer drawer,
			final boolean prepend) throws IOException {

//		long t1 = System.currentTimeMillis();
		if (this.mPending.contains(cacheURL))
			return true;

		if (mode == TileProvider.MODE_CONNECTED)
			return false;

		// final String formattedTileURLString =
		// TileCache.format(aTileURLString);
		final String quadDir = strategy.getRelativeToLayerDirTilePath(
				tile.tile, tile.zoomLevel);

		String st = getSDDir();
		final File f = new File(new StringBuffer().append(st).append(
				File.separator).append(Constants.APP_DIR)
				.append(File.separator).append(Constants.MAPS_DIR).append(
						File.separator).append(tile.layerName).append(
						strategy.getLayerNameSuffix()).append(File.separator)
				.append(quadDir).append(strategy.getTileNameSuffix())
				.toString());

		if (!f.exists()) {
			if (mode == TileProvider.MODE_OFFLINE
					|| mode == TileProvider.MODE_DELETE) {
				// context.sendMessage(callback, aTileURLString,
				// MAPTILEFSLOADER_NOTFOUND_ID);
				context.sendMessage(callback, this.instantiateEvent(tile, ""),
						MAPTILEFSLOADER_SUCCESS_ID);
				TileFilesystemProvider.this.mPending.remove(cacheURL);
//				long t2 = System.currentTimeMillis();
//				System.out.println("time to check if exists: " + (t2 - t1)
//						+ ".ms");
				// FIXME: true or false?
				return false;
			} else {
//				long t2 = System.currentTimeMillis();
//				System.out.println("time to check if exists: " + (t2 - t1)
//						+ ".ms");
				return false;
			}
		} else {
			if (f.length() <= 0 || mode == TileProvider.MODE_DELETE) {
				f.delete();
				if (mode == TileProvider.MODE_DELETE) {
					context.sendMessage(callback, this.instantiateEvent(tile, f
							.getPath()), MAPTILEFSLOADER_DELETED_ID);
				}
//				long t2 = System.currentTimeMillis();
//				System.out.println("void file: " + (t2 - t1) + ".ms");
				return false;
			} else if (mode == TileProvider.MODE_UPDATE) {
//				long t2 = System.currentTimeMillis();
//				System.out.println("mode update: " + (t2 - t1) + ".ms");
				return false;
			}
		}

		final FileInputStream in = new FileInputStream(f);

		this.mPending.add(cacheURL);

		es.prodevelop.gvsig.mini.utiles.WorkQueue.getInstance().execute(
				new Runnable() {
					// @Override
					public void run() {
						OutputStream out = null;
						ByteArrayOutputStream dataStream = null;

						boolean succeed = false;
						int retryCount = 0;

						while (retryCount <= TileProvider.NUM_RETRY && !succeed) {
							try {
								dataStream = new ByteArrayOutputStream();

								out = new BufferedOutputStream(dataStream,
										Constants.IO_BUFFER_SIZE);
								Constants.copy(in, out);
								out.flush();

								final byte[] data = dataStream.toByteArray();
								if (data.length <= 0) {
									throw new IOException("");
								}

								if (cancellable.getCanceled()) {
									mPending.clear();
									WorkQueue.getInstance().clearPendingTasks();
									context.sendMessage(callback,
											TileFilesystemProvider.this
													.instantiateEvent(tile, f
															.getPath()),
											GeoUtils.TILEPROVIDER_CANCELED);
									return;
								}

								final IBitmap bmp = context.decodeByteArray(
										data, 0, data.length);

								if (TileFilesystemProvider.this.mCache != null)
									TileFilesystemProvider.this.mCache.putTile(
										cacheURL, bmp);

								context.sendMessage(callback,
										TileFilesystemProvider.this
												.instantiateEvent(tile, f
														.getPath()),
										MAPTILEFSLOADER_SUCCESS_ID);
								succeed = true;

								if (drawer != null)
									drawer.drawTile(tile.clone(), bmp);

								// log.log(Level.FINE,"Loaded from disk: " +
								// aTileURLString);
							} catch (java.lang.OutOfMemoryError ex) {
								System.gc();
								log.log(Level.SEVERE, "OutOfMemoryError: ", ex);
								TileFilesystemProvider.this.mCache
										.onLowMemory();
								if (retryCount == TileProvider.NUM_RETRY)
									context.sendMessage(callback,
											TileFilesystemProvider.this
													.instantiateEvent(tile, f
															.getPath()),
											MAPTILEFSLOADER_OOM_ID);
							} catch (Exception e) {
								if (retryCount == TileProvider.NUM_RETRY)
									context.sendMessage(callback,
											TileFilesystemProvider.this
													.instantiateEvent(tile, f
															.getPath()),
											MAPTILEFSLOADER_FAIL_ID);
							} finally {
								Constants.closeStream(in);
								Constants.closeStream(dataStream);
								Constants.closeStream(out);
								retryCount++;
							}
						}
						if (mode == TileProvider.MODE_OFFLINE && !succeed) {
							TileFilesystemProvider.this.mCache.putTile(
									cacheURL, offlineBitmap);
							context
									.sendMessage(callback,
											TileFilesystemProvider.this
													.instantiateEvent(tile, f
															.getPath()),
											MAPTILEFSLOADER_SUCCESS_ID);
							TileFilesystemProvider.this.mPending
									.remove(cacheURL);
						}
						TileFilesystemProvider.this.mPending.remove(cacheURL);
					}
				}, prepend);
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * es.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider#saveFile
	 * (java.lang.String, byte[], int[], java.lang.String, int)
	 */
	public synchronized void saveFile(final byte[] someData, final Tile tile)
			throws IOException {
		// final String filename = TileCache.format(aURLString);
		if (mode == TileProvider.MODE_CONNECTED)
			return;

		final String quadDir = strategy.getRelativeToLayerDirTilePath(
				tile.tile, tile.zoomLevel);

		String st = getSDDir();
		final File f = new File(new StringBuffer(st).append(File.separator)
				.append(Constants.APP_DIR).append(File.separator).append(
						Constants.MAPS_DIR).append(File.separator).append(
						tile.layerName).append(strategy.getLayerNameSuffix())
				.append(File.separator).append(quadDir).append(
						strategy.getTileNameSuffix()).toString());

		if (f.exists()) {
			if (f.length() <= 0) {
				// log.log(Level.FINE, "Deleting invalid file");
				f.delete();
				f.createNewFile();
			}
		} else {
			// String path = f.getAbsolutePath();
			// path = path.substring(0, path.lastIndexOf("/"));
			// final File f1 = new File(path);
			// log.log(Level.FINE, "mkdirs: ");
			File dir = new File(f.getParent());
			if (dir.mkdirs()) {
				// log.log(Level.FINE, "directories created: " + aURLString);
			} else {
				if (f.getParent() == null) {
					// log.log(Level.FINE, "parent = null");
				}
				// log.log(Level.FINE, "directories failed " + aURLString);
			}

			// if (f.getParentFile().exists() && f.createNewFile()) {
			// log.log(Level.FINE, "new file created " + aURLString);
			// } else {
			// log.log(Level.FINE, "new file failed " + aURLString);
			// }
			// path = null;

		}
		final FileOutputStream fos = new FileOutputStream(f);
		final BufferedOutputStream bos = new BufferedOutputStream(fos,
				Constants.IO_BUFFER_SIZE);

		try {
			// log.log(Level.FINE, "prepared to write " + aURLString);
			bos.write(someData);
			bos.flush();
			bos.close();
			// log.log(Level.FINE, "Tile stored " + aURLString);
		} catch (Exception e) {
			log.log(Level.SEVERE, "save file", e);
		}

		Constants.closeStream(fos);
		Constants.closeStream(bos);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * es.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider#destroy
	 * ()
	 */
	public void destroy() {
		try {

		} catch (Exception e) {
			log.log(Level.SEVERE, "", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seees.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider#
	 * getStrategy()
	 */
	public ITileFileSystemStrategy getStrategy() {
		return strategy;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seees.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider#
	 * setStrategy
	 * (es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy
	 * )
	 */
	public void setStrategy(ITileFileSystemStrategy strategy) {
		this.strategy = strategy;
	}

	public TileCache getTileCache() {
		// TODO Auto-generated method stub
		return mCache;
	}

	public Set getPendingQueue() {
		return mPending;
	}

	public void setMode(int mode) {
		this.mode = mode;
	}

	public int getMode() {
		return mode;
	}

	protected TileEvent instantiateEvent(Tile aTile, String path) {
		return new TileEvent(aTile.clone(), path, getMode(), getStrategy()
				.getName(), TileEvent.TILE_LEVEL_FILE_SYSTEM);
	}
}
