/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integraci�n de Tecnolog�as SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache.provider.filesystem.strategy;

/**
 * This interface allows to save/load tiles attending to different strategies, this means
 * the way the tiles are stored on disk, using a quadkey, flat zoom level, etc.
 *  
 * @author aromeu
 *
 */
public interface ITileFileSystemStrategy {
	
	public static final String FLATX = "flatx";
	public static final String QUADKEY = "quadkey";
	
	/**
	 * The relative path of the tile on disk
	 * @param tile tile x-y
	 * @param zoomLevel the zoomlevel
	 * @return The path to append to the absolute path of the layer on disk
	 */
	public String getRelativeToLayerDirTilePath(int[] tile, int zoomLevel);
	
	/**
	 * This method return a suffix to append to the layerName in order to have
	 * separate directories for different strategies of the same layers
	 * @return The suffix
	 */
	public String getLayerNameSuffix();

	/**
	 * This method return a suffix to append to the tile Name in order to allow
	 * the application load tiles downloaded from other providers with a random
	 * suffix
	 * @return The suffix
	 */	 
	public String getTileNameSuffix();
	
	/**
	 * This method sets a suffix to append to the tile Name in order to allow
	 * the application load tiles downloaded from other providers with a random
	 * suffix
	 * 
	 */	 
	public void setTileNameSuffix(String suffix);
	
	/**
	 * The description of the strategy
	 * @return
	 */
	public String getName();
}
