/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Alberto Romeu aromeu@prodevelop.es 
 *   author Ruben Blanco rblanco@prodevelop.es 
 *   
 */

package es.prodevelop.tilecache.renderer;

import java.util.logging.Level;
import java.util.logging.Logger;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Point;

/**
 * A Factory of MapRenderers
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class MapRendererFactory implements IMapRendererFactory {

	private final static Logger logger = Logger
			.getLogger("MapRendererFactory");

	/**
	 * Instantiates a MapRenderer. @see Layers and layers.txt
	 * 
	 * @param layerTitle
	 *            The name of the MapRenderer
	 * @param layerProps
	 *            The properties of the MapRenderer
	 * @return
	 */
	public MapRenderer getMapRenderer(String layerTitle, String[] layerProps) {
		MapRenderer renderer = null;
		try {
			int index = layerProps[0].indexOf("[");
			Extent offExtent = null;
			String offlineLayerName = "";
			if (index != -1) {
				try {
					String ex = layerProps[0].substring(index + 1, layerProps[0].length() - 1);
					if (ex != null && ex.compareTo("") != 0) {
						String[] parts = ex.split(MapRenderer.NAME_SEPARATOR);						
						offExtent = Extent.parseString(parts[0], ":");
						offlineLayerName = parts[1];
					}
				} catch (Exception e) {
					
				}
				
			}
				
			int type = Integer.valueOf(layerProps[0].substring(0, 1)).intValue();		
			String[] url = layerProps[1].split(MapRenderer.URL_SEPARATOR);
			String format = layerProps[2];
			int minZoomLevel = Integer.valueOf(layerProps[4]).intValue();
			int maxZoomLevel = Integer.valueOf(layerProps[3]).intValue();
			int tileSize = Integer.valueOf(layerProps[5]).intValue();
			double minX = Double.valueOf(layerProps[8]).doubleValue();
			double minY = Double.valueOf(layerProps[9]).doubleValue();
			double maxX = Double.valueOf(layerProps[10]).doubleValue();
			double maxY = Double.valueOf(layerProps[11]).doubleValue();
			double originX = Double.valueOf(layerProps[6]).doubleValue();
			double originY = Double.valueOf(layerProps[7]).doubleValue();
			String SRS = layerProps[12];

			String[] resol = layerProps[13].split(":");
			final int length = resol.length;
			double[] resolutions = new double[length];
			for (int i = 0; i < length; i++) {
				resolutions[i] = Double.valueOf(resol[i]).doubleValue();
			}			
			
			switch (type) {
			case MapRenderer.EQUATOR_RENDERER:
				logger.log(Level.FINE, "EQUATOR_RENDERER " + layerTitle);
				renderer = EquatorRenderer.getEquatorRenderer(url,
						layerTitle, format, maxZoomLevel, minZoomLevel, tileSize,
						type, new Point(originX, originY), new Extent(minX, minY,
								maxX, maxY), SRS, resolutions, offExtent);				
				break;
			case MapRenderer.OSM_RENDERER:
				logger.log(Level.FINE, "OSM_RENDERER " + layerTitle);
				renderer = OSMMercatorRenderer.getOSMMercatorRenderer(url,
						layerTitle, format, maxZoomLevel, minZoomLevel, tileSize,
						type, new Point(originX, originY), new Extent(minX, minY,
								maxX, maxY), SRS, resolutions, offExtent);
				break;
			case MapRenderer.OSM_ZYX_RENDERER:
				logger.log(Level.FINE, "OSM_ZXY_RENDERER " + layerTitle);
				renderer = OSMZYXRenderer.getOSMZYXRenderer(url,
						layerTitle, format, maxZoomLevel, minZoomLevel, tileSize,
						type, new Point(originX, originY), new Extent(minX, minY,
								maxX, maxY), SRS, resolutions, offExtent);
				break;
			case MapRenderer.OSMPARMS_RENDERER:
				logger.log(Level.FINE, "OSMPARMS_RENDERER " + layerTitle);
				renderer = OSMParmsRenderer.getOSMParmsRenderer(url,
						layerTitle, format, maxZoomLevel, minZoomLevel, tileSize,
						type, new Point(originX, originY), new Extent(minX, minY,
								maxX, maxY), SRS, resolutions, offExtent);
				break;
			case MapRenderer.QUADKEY_RENDERER:
				logger.log(Level.FINE, "QUADKEY_RENDERER " + layerTitle);
				renderer = QuadkeyRenderer.getQuadkeyRenderer(url,
						layerTitle, format, maxZoomLevel, minZoomLevel, tileSize,
						type, new Point(originX, originY), new Extent(minX, minY,
								maxX, maxY), SRS, resolutions, offExtent);
				break;
			case MapRenderer.QRST_RENDERER:
				logger.log(Level.FINE, "QRST_RENDERER " + layerTitle);
				renderer = QRSTRenderer.getQRSTRenderer(url,
						layerTitle, format, maxZoomLevel, minZoomLevel, tileSize,
						type, new Point(originX, originY), new Extent(minX, minY,
								maxX, maxY), SRS, resolutions, offExtent);
				break;
			case MapRenderer.TMS_RENDERER:
				logger.log(Level.FINE, "Found a TMS layer" + layerTitle);
				renderer = TMSRenderer.getTMSRenderer(layerProps, layerTitle);
				break;
			case MapRenderer.WMS_RENDERER:
				logger.log(Level.FINE, "Found a WMS layer: " + layerTitle);
				// renderer = WMSRenderer
				// .getWMSRenderer(layerProps, layerTitle);
				break;
			}

			if (renderer != null &&offlineLayerName != null)
				renderer.setOfflineLayerName(offlineLayerName);
			return renderer;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getMapRenderer: ", e);
			return renderer;
		}
	}
}
