package es.prodevelop.tilecache.summary;

import java.util.Hashtable;
import java.util.Iterator;

import es.prodevelop.gvsig.mini.common.CompatManager;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.tilecache.layers.Layers;
import es.prodevelop.tilecache.renderer.MapRenderer;
import es.prodevelop.tilecache.renderer.MapRendererManager;

/**
 * The info that summary writer needs to persist
 * 
 * @author aromeu
 * 
 */
public class SummaryInfo {

	private MapRenderer renderer;
	private int minZoomLevel;
	private int maxZoomLevel;
	private Extent downloadedExtent;
	private Extent rendererExtent;
	private String cityName;

	public final static String FILE_NAME = "layer.txt";

	public final static String TILES_DOWN = "tiles_downloaded";
	public final static String TILES_FAILED = "tiles_failed";
	public final static String TILES_SKIPPED = "tiles_skipped";
	public final static String TILES_FROM_FS = "tiles_fromFS";
	public final static String TILES_NOT_FOUND = "tiles_not_found";
	public final static String TILES_DELETED = "tiles_deleted";
	public final static String BYTES_DOWNLOADED = "bytes_downloaded";
	public final static String TIME_TO_DOWNLOAD = "time_to_download";

	public final static String MIN_ZOOM_LEVEL = "minZ";
	public final static String MAX_ZOOM_LEVEL = "maxZ";
	public final static String EXTENT = "downloaded_extent";
	public final static String NAME = "city_name";

	private Hashtable<String, String> props = new Hashtable<String, String>();

	private String version;
	private String layerNameSuffix;

	/**
	 * Uses this constructor when reading the SummaryInfo from a persisted file
	 * 
	 * @param renderer
	 * @param props
	 * @throws Exception
	 */
	public SummaryInfo(String renderer, Hashtable<String, String> props)
			throws Exception {
		String[] rend = renderer.split("\\;");
		this.renderer = MapRendererManager.getInstance()
				.getMapRendererFactory().getMapRenderer(rend[0],
						rend[1].split(","));

		Iterator<String> it = props.keySet().iterator();
		while (it.hasNext()) {
			String next = it.next();			
			this.props.put(next, props.get(next));
		}

	}

	/**
	 * Use this constructor to write the summary info
	 */
	public SummaryInfo(MapRenderer renderer, int minZoomLevel,
			int maxZoomLevel, Extent downloadedExtent, String version,
			String layerNameSuffix, String cityName, Extent rendererExtent) {
		this.renderer = renderer;
		this.minZoomLevel = minZoomLevel;
		this.maxZoomLevel = maxZoomLevel;
		this.version = version;
		this.downloadedExtent = downloadedExtent;
		renderer.setZoomMinLevel(minZoomLevel);
		renderer.setZOOM_MAXLEVEL(maxZoomLevel);
		this.rendererExtent = rendererExtent;
//		renderer.setExtent(rendererExtent);
		this.layerNameSuffix = layerNameSuffix;
		this.cityName = cityName;

		fillProperties();
	}

	private void fillProperties() {
		this.put(MIN_ZOOM_LEVEL, String.valueOf(minZoomLevel));
		this.put(MAX_ZOOM_LEVEL, String.valueOf(maxZoomLevel));
		this.put(EXTENT, downloadedExtent.toString());
		this.put(NAME, cityName.toString());
	}

	public Hashtable<String, String> getProperties() {
		return this.props;
	}

	public void put(String key, String value) {
		this.props.put(key, value);
	}

	public String getValue(String key) {
		return props.get(key);
	}

	public MapRenderer getRenderer() {
		return renderer;
	}

	public void setRenderer(MapRenderer renderer) {
		this.renderer = renderer;
	}

	public int getMinZoomLevel() {
		return minZoomLevel;
	}

	public void setMinZoomLevel(int minZoomLevel) {
		this.minZoomLevel = minZoomLevel;
	}

	public int getMaxZoomLevel() {
		return maxZoomLevel;
	}

	public void setMaxZoomLevel(int maxZoomLevel) {
		this.maxZoomLevel = maxZoomLevel;
	}

	public Extent getDownloadedExtent() {
		return downloadedExtent;
	}

	public void setDownloadedExtent(Extent downloadedExtent) {
		this.downloadedExtent = downloadedExtent;
	}

	public String getVersion() {
		return this.version;
	}

	public String getLayerNameSuffix() {
		return this.layerNameSuffix;
	}
	
	public String getCityName() {
		return this.cityName;
	}
	
	public void setCityName(String cityName) {
		this.cityName = cityName;
	}
	
	public Extent getRendererExtent() {
		return rendererExtent;
	}

}
